Block-Structured AMR Software Framework
Loading...
Searching...
No Matches
AMReX_BoxArray.H
Go to the documentation of this file.
1
2#ifndef BL_BOXARRAY_H
3#define BL_BOXARRAY_H
4#include <AMReX_Config.H>
5
6#include <AMReX_IndexType.H>
7#include <AMReX_BoxList.H>
8#include <AMReX_Array.H>
9#include <AMReX_Periodicity.H>
10#include <AMReX_Vector.H>
11
12#include <iosfwd>
13#include <atomic>
14#include <cstddef>
15#include <cstdint>
16#include <map>
17#include <memory>
18#include <unordered_map>
19
25namespace amrex
26{
27 class BoxArray;
28
36 [[nodiscard]] BoxArray boxComplement (const Box& b1in, const Box& b2);
37
45 [[nodiscard]] BoxArray complementIn (const Box& b, const BoxArray& ba);
46
54 [[nodiscard]] BoxArray intersect (const BoxArray& ba, const Box& b, int ng = 0);
55
63 [[nodiscard]] BoxArray intersect (const BoxArray& ba, const Box& b, const IntVect& ng);
64
66 [[nodiscard]] BoxArray intersect (const BoxArray& lhs, const BoxArray& rhs);
67
69 [[nodiscard]] BoxList intersect (const BoxArray& ba, const BoxList& bl);
70
80 [[nodiscard]] BoxArray convert (const BoxArray& ba, IndexType typ);
90 [[nodiscard]] BoxArray convert (const BoxArray& ba, const IntVect& typ);
91
101 [[nodiscard]] BoxArray coarsen (const BoxArray& ba, int ratio);
111 [[nodiscard]] BoxArray coarsen (const BoxArray& ba, const IntVect& ratio);
112
122 [[nodiscard]] BoxArray refine (const BoxArray& ba, int ratio);
132 [[nodiscard]] BoxArray refine (const BoxArray& ba, const IntVect& ratio);
133
146 [[nodiscard]] BoxList GetBndryCells (const BoxArray& ba, int ngrow);
147
155 void readBoxArray (BoxArray& ba, std::istream& s, bool b = false);
156
167 [[nodiscard]] bool match (const BoxArray& x, const BoxArray& y);
168
184 [[nodiscard]] BoxArray decompose (Box const& domain, int nboxes,
185 Array<bool,AMREX_SPACEDIM> const& decomp
186 = {AMREX_D_DECL(true,true,true)},
187 bool no_overlap = false);
188
190struct BARef
191{
192 BARef ();
193 explicit BARef (size_t size);
194 explicit BARef (const Box& b);
195 explicit BARef (const BoxList& bl);
196 explicit BARef (BoxList&& bl) noexcept;
197 explicit BARef (std::istream& is);
198 BARef (const BARef& rhs);
199 BARef (BARef&& rhs) = delete;
200 BARef& operator= (const BARef& rhs) = delete;
201 BARef& operator= (BARef&& rhs) = delete;
202
203 ~BARef ();
204
205 void define (const Box& bx);
206 void define (const BoxList& bl);
207 void define (BoxList&& bl) noexcept;
208 void define (std::istream& is, int& ndims);
210 void resize (Long n);
211#ifdef AMREX_MEM_PROFILING
212 void updateMemoryUsage_box (int s);
213 void updateMemoryUsage_hash (int s);
214#endif
215
216 [[nodiscard]] bool HasHashMap () const noexcept {
217 return has_hashmap.load(std::memory_order_acquire);
218 }
219
220 //
222 Vector<Box> m_abox;
223 //
225 mutable Box bbox;
226
227 mutable IntVect crsn;
228
229 using HashType = std::unordered_map< IntVect, std::vector<int>, IntVect::shift_hasher > ;
230 //using HashType = std::map< IntVect,std::vector<int> >;
231
232 mutable HashType hash;
233
234 mutable std::atomic<bool> has_hashmap = false;
235
236 static int numboxarrays;
237 static int numboxarrays_hwm;
238 static Long total_box_bytes;
239 static Long total_box_bytes_hwm;
240 static Long total_hash_bytes;
241 static Long total_hash_bytes_hwm;
242
243 static void Initialize ();
244 static void Finalize ();
245 static bool initialized;
246};
248
250struct BATnull
251{
252 [[nodiscard]] Box operator() (const Box& bx) const noexcept { return bx; }
253 [[nodiscard]] static constexpr Box coarsen (Box const& a_box) { return a_box; }
254 [[nodiscard]] static constexpr IntVect doiLo () { return IntVect::TheZeroVector(); }
255 [[nodiscard]] static constexpr IntVect doiHi () { return IntVect::TheZeroVector(); }
256 [[nodiscard]] static constexpr IndexType index_type () { return IndexType(); }
257 [[nodiscard]] static constexpr IntVect coarsen_ratio () { return IntVect::TheUnitVector(); }
258};
260
262struct BATindexType
263{
264 explicit BATindexType (IndexType a_typ) : m_typ(a_typ) {}
265 [[nodiscard]] Box operator() (const Box& bx) const noexcept { return amrex::convert(bx,m_typ); }
266 [[nodiscard]] static Box coarsen (Box const& a_box) noexcept { return a_box; }
267 [[nodiscard]] static constexpr IntVect doiLo () { return IntVect::TheZeroVector(); }
268 [[nodiscard]] IntVect doiHi () const noexcept { return m_typ.ixType(); }
269 [[nodiscard]] IndexType index_type () const noexcept { return m_typ; }
270 [[nodiscard]] static constexpr IntVect coarsen_ratio () { return IntVect::TheUnitVector(); }
271 IndexType m_typ;
272};
274
276struct BATcoarsenRatio
277{
278 explicit BATcoarsenRatio (IntVect const& a_crse_ratio) : m_crse_ratio(a_crse_ratio) {}
279 [[nodiscard]] Box operator() (const Box& bx) const noexcept { return amrex::coarsen(bx,m_crse_ratio); }
280 [[nodiscard]] Box coarsen (Box const& a_box) const noexcept { return amrex::coarsen(a_box,m_crse_ratio); }
281 [[nodiscard]] static constexpr IntVect doiLo () { return IntVect::TheZeroVector(); }
282 [[nodiscard]] static constexpr IntVect doiHi () { return IntVect::TheZeroVector(); }
283 [[nodiscard]] static constexpr IndexType index_type () { return IndexType(); }
284 [[nodiscard]] IntVect coarsen_ratio () const noexcept { return m_crse_ratio; }
285 IntVect m_crse_ratio;
286};
288
290struct BATindexType_coarsenRatio
291{
292 BATindexType_coarsenRatio (IndexType a_typ, IntVect const& a_crse_ratio)
293 : m_typ(a_typ), m_crse_ratio(a_crse_ratio) {}
294
295 [[nodiscard]] Box operator() (const Box& bx) const noexcept {
296 return amrex::convert(amrex::coarsen(bx,m_crse_ratio),m_typ);
297 }
298
299 [[nodiscard]] Box coarsen (Box const& a_box) const noexcept { return amrex::coarsen(a_box,m_crse_ratio); }
300
301 [[nodiscard]] static constexpr IntVect doiLo () { return IntVect::TheZeroVector(); }
302 [[nodiscard]] IntVect doiHi () const noexcept { return m_typ.ixType(); }
303
304 [[nodiscard]] IndexType index_type () const noexcept { return m_typ; }
305 [[nodiscard]] IntVect coarsen_ratio () const noexcept { return m_crse_ratio; }
306
307 IndexType m_typ;
308 IntVect m_crse_ratio;
309};
311
313struct BATbndryReg
314{
315 BATbndryReg (Orientation a_face, IndexType a_typ,
316 int a_in_rad, int a_out_rad, int a_extent_rad)
317 : m_face(a_face), m_typ(a_typ), m_crse_ratio(1)
318 {
319 m_loshft = IntVect(-a_extent_rad);
320 m_hishft = IntVect( a_extent_rad);
321 IntVect nodal = a_typ.ixType();
322 m_hishft += nodal;
323 const int d = a_face.coordDir();
324 if (nodal[d]) {
325 // InterpFaceRegister & SyncRegister in IAMR
326 if (m_face.isLow()) {
327 m_loshft[d] = 0;
328 m_hishft[d] = 0;
329 } else {
330 m_loshft[d] = 1;
331 m_hishft[d] = 1;
332 }
333 m_doilo = IntVect(0);
334 m_doihi = nodal;
335 } else {
336 // BndryRegister
337 if (m_face.isLow()) {
338 m_loshft[d] = nodal[d] - a_out_rad;
339 m_hishft[d] = nodal[d] + a_in_rad - 1;
340 } else {
341 m_loshft[d] = 1 - a_in_rad;
342 m_hishft[d] = a_out_rad;
343 }
344 m_doilo = IntVect(a_extent_rad);
345 m_doihi = IntVect(a_extent_rad);
346 m_doihi += nodal;
347 if (m_face.isLow()) { // domain of influence in index space
348 m_doilo[d] = a_out_rad;
349 m_doihi[d] = 0;
350 } else {
351 m_doilo[d] = 0;
352 m_doihi[d] = a_out_rad;
353 }
354 }
355 }
356
357 [[nodiscard]] Box operator() (const Box& a_bx) const noexcept {
358 IntVect lo = amrex::coarsen(a_bx.smallEnd(), m_crse_ratio);
359 IntVect hi = amrex::coarsen(a_bx.bigEnd(), m_crse_ratio);
360 const int d = m_face.coordDir();
361 if (m_face.isLow()) {
362 hi[d] = lo[d];
363 } else {
364 lo[d] = hi[d];
365 }
366 lo += m_loshft;
367 hi += m_hishft;
368 return Box(lo,hi,m_typ);
369 }
370
371 [[nodiscard]] Box coarsen (Box const& a_box) const noexcept { return amrex::coarsen(a_box,m_crse_ratio); }
372
373 [[nodiscard]] IntVect doiLo () const noexcept { return m_doilo; }
374 [[nodiscard]] IntVect doiHi () const noexcept { return m_doihi; }
375
376 [[nodiscard]] IndexType index_type () const noexcept { return m_typ; }
377 [[nodiscard]] IntVect coarsen_ratio () const noexcept { return m_crse_ratio; }
378
379 friend bool operator== (BATbndryReg const& a, BATbndryReg const& b) noexcept {
380 return a.m_face == b.m_face && a.m_typ == b.m_typ && a.m_crse_ratio == b.m_crse_ratio
381 && a.m_loshft == b.m_loshft && a.m_hishft == b.m_hishft
382 && a.m_doilo == b.m_doilo && a.m_doihi == b.m_doihi;
383 }
384
385 Orientation m_face;
386 IndexType m_typ;
387 IntVect m_crse_ratio;
388 IntVect m_loshft;
389 IntVect m_hishft;
390 IntVect m_doilo;
391 IntVect m_doihi;
392};
394
396struct BATransformer
397{
398 enum struct BATType : std::int8_t { null, indexType, coarsenRatio, indexType_coarsenRatio, bndryReg};
399
400 union BATOp {
401 BATOp () noexcept
402 : m_null() {}
403 BATOp (IndexType t) noexcept
404 : m_indexType(t) {}
405 BATOp (IntVect const& r) noexcept
406 : m_coarsenRatio(r) {}
407 BATOp (IndexType t, IntVect const& r) noexcept
408 : m_indexType_coarsenRatio(t,r) {}
409 BATOp (Orientation f, IndexType t, int in_rad, int out_rad, int extent_rad) noexcept
410 : m_bndryReg(f,t,in_rad,out_rad,extent_rad) {}
411 BATnull m_null;
412 BATindexType m_indexType;
413 BATcoarsenRatio m_coarsenRatio;
414 BATindexType_coarsenRatio m_indexType_coarsenRatio;
415 BATbndryReg m_bndryReg;
416 };
417
418 BATransformer () = default;
419
420 BATransformer (IndexType t)
421 : m_bat_type(t.cellCentered() ? BATType::null : BATType::indexType),
422 m_op (t.cellCentered() ? BATOp() : BATOp(t)) {}
423
424 BATransformer (Orientation f, IndexType t, int in_rad, int out_rad, int extent_rad)
425 : m_bat_type(BATType::bndryReg),
426 m_op(f,t,in_rad,out_rad,extent_rad) {}
427
428 [[nodiscard]] Box operator() (Box const& ab) const noexcept {
429 switch (m_bat_type)
430 {
431 case BATType::null:
432 return m_op.m_null(ab);
433 case BATType::indexType:
434 return m_op.m_indexType(ab);
435 case BATType::coarsenRatio:
436 return m_op.m_coarsenRatio(ab);
437 case BATType::indexType_coarsenRatio:
438 return m_op.m_indexType_coarsenRatio(ab);
439 default:
440 return m_op.m_bndryReg(ab);
441 }
442 }
443
444 [[nodiscard]] Box coarsen (Box const& a_box) const noexcept {
445 switch (m_bat_type)
446 {
447 case BATType::null:
448 return amrex::BATnull::coarsen(a_box);
449 case BATType::indexType:
450 return amrex::BATindexType::coarsen(a_box);
451 case BATType::coarsenRatio:
452 return m_op.m_coarsenRatio.coarsen(a_box);
453 case BATType::indexType_coarsenRatio:
454 return m_op.m_indexType_coarsenRatio.coarsen(a_box);
455 default:
456 return m_op.m_bndryReg.coarsen(a_box);
457 }
458 }
459
460 [[nodiscard]] IntVect doiLo () const noexcept {
461 switch (m_bat_type)
462 {
463 case BATType::null:
464 return amrex::BATnull::doiLo();
465 case BATType::indexType:
466 return amrex::BATindexType::doiLo();
467 case BATType::coarsenRatio:
468 return amrex::BATcoarsenRatio::doiLo();
469 case BATType::indexType_coarsenRatio:
470 return amrex::BATindexType_coarsenRatio::doiLo();
471 default:
472 return m_op.m_bndryReg.doiLo();
473 }
474 }
475
476 [[nodiscard]] IntVect doiHi () const noexcept {
477 switch (m_bat_type)
478 {
479 case BATType::null:
480 return amrex::BATnull::doiHi();
481 case BATType::indexType:
482 return m_op.m_indexType.doiHi();
483 case BATType::coarsenRatio:
484 return amrex::BATcoarsenRatio::doiHi();
485 case BATType::indexType_coarsenRatio:
486 return m_op.m_indexType_coarsenRatio.doiHi();
487 default:
488 return m_op.m_bndryReg.doiHi();
489 }
490 }
491
492 [[nodiscard]] IndexType index_type () const noexcept {
493 switch (m_bat_type)
494 {
495 case BATType::null:
496 return amrex::BATnull::index_type();
497 case BATType::indexType:
498 return m_op.m_indexType.index_type();
499 case BATType::coarsenRatio:
500 return amrex::BATcoarsenRatio::index_type();
501 case BATType::indexType_coarsenRatio:
502 return m_op.m_indexType_coarsenRatio.index_type();
503 default:
504 return m_op.m_bndryReg.index_type();
505 }
506 }
507
508 [[nodiscard]] IntVect coarsen_ratio () const noexcept {
509 switch (m_bat_type)
510 {
511 case BATType::null:
512 return amrex::BATnull::coarsen_ratio();
513 case BATType::indexType:
514 return amrex::BATindexType::coarsen_ratio();
515 case BATType::coarsenRatio:
516 return m_op.m_coarsenRatio.coarsen_ratio();
517 case BATType::indexType_coarsenRatio:
518 return m_op.m_indexType_coarsenRatio.coarsen_ratio();
519 default:
520 return m_op.m_bndryReg.coarsen_ratio();
521 }
522 }
523
524 [[nodiscard]] bool is_null () const noexcept {
525 return m_bat_type == BATType::null;
526 }
527
528 [[nodiscard]] bool is_simple () const noexcept {
529 return m_bat_type != BATType::bndryReg;
530 }
531
532 void set_coarsen_ratio (IntVect const& a_ratio) noexcept {
533 switch (m_bat_type)
534 {
535 case BATType::null:
536 {
537 if (a_ratio == IntVect::TheUnitVector()) {
538 return;
539 } else {
540 m_bat_type = BATType::coarsenRatio;
541 m_op.m_coarsenRatio.m_crse_ratio = a_ratio;
542 return;
543 }
544 }
545 case BATType::indexType:
546 {
547 if (a_ratio == IntVect::TheUnitVector()) {
548 return;
549 } else {
550 m_bat_type = BATType::indexType_coarsenRatio;
551 auto t = m_op.m_indexType.m_typ;
552 m_op.m_indexType_coarsenRatio.m_typ = t;
553 m_op.m_indexType_coarsenRatio.m_crse_ratio = a_ratio;
554 return;
555 }
556 }
557 case BATType::coarsenRatio:
558 {
559 if (a_ratio == IntVect::TheUnitVector()) {
560 m_bat_type = BATType::null;
561 return;
562 } else {
563 m_op.m_coarsenRatio.m_crse_ratio = a_ratio;
564 return;
565 }
566 }
567 case BATType::indexType_coarsenRatio:
568 {
569 if (a_ratio == IntVect::TheUnitVector()) {
570 m_bat_type = BATType::indexType;
571 auto t = m_op.m_indexType_coarsenRatio.m_typ;
572 m_op.m_indexType.m_typ = t;
573 return;
574 } else {
575 m_op.m_indexType_coarsenRatio.m_crse_ratio = a_ratio;
576 return;
577 }
578 }
579 default:
580 {
581 m_op.m_bndryReg.m_crse_ratio = a_ratio;
582 return;
583 }
584 }
585 }
586
587 void set_index_type (IndexType typ) noexcept {
588 switch (m_bat_type)
589 {
590 case BATType::null:
591 {
592 if (typ.cellCentered()) {
593 return;
594 } else {
595 m_bat_type = BATType::indexType;
596 m_op.m_indexType.m_typ = typ;
597 return;
598 }
599 }
600 case BATType::indexType:
601 {
602 if (typ.cellCentered()) {
603 m_bat_type = BATType::null;
604 return;
605 } else {
606 m_op.m_indexType.m_typ = typ;
607 return;
608 }
609 }
610 case BATType::coarsenRatio:
611 {
612 if (typ.cellCentered()) {
613 return;
614 } else {
615 m_bat_type = BATType::indexType_coarsenRatio;
616 auto r = m_op.m_coarsenRatio.m_crse_ratio;
617 m_op.m_indexType_coarsenRatio.m_typ = typ;
618 m_op.m_indexType_coarsenRatio.m_crse_ratio = r;
619 return;
620 }
621 }
622 case BATType::indexType_coarsenRatio:
623 {
624 if (typ.cellCentered()) {
625 m_bat_type = BATType::coarsenRatio;
626 auto r = m_op.m_indexType_coarsenRatio.m_crse_ratio;
627 m_op.m_coarsenRatio.m_crse_ratio = r;
628 return;
629 } else {
630 m_op.m_indexType_coarsenRatio.m_typ = typ;
631 return;
632 }
633 }
634 default:
635 {
636 m_op.m_bndryReg.m_typ = typ;
637 return;
638 }
639 }
640 }
641
642 friend bool operator== (BATransformer const& a, BATransformer const& b) noexcept {
643 if (a.m_bat_type != BATType::bndryReg && b.m_bat_type != BATType::bndryReg) {
644 return a.index_type() == b.index_type()
645 && a.coarsen_ratio() == b.coarsen_ratio();
646 } else if (a.m_bat_type == BATType::bndryReg && b.m_bat_type == BATType::bndryReg) {
647 return a.m_op.m_bndryReg == b.m_op.m_bndryReg;
648 } else {
649 return false;
650 }
651 }
652
653 BATType m_bat_type{BATType::null};
654 BATOp m_op;
655};
657
658// for backward compatibility
659using BndryBATransformer = BATransformer;
660
661class MFIter;
662class AmrMesh;
663class FabArrayBase;
664
676{
677public:
678
680 BoxArray () noexcept;
681 BoxArray (const BoxArray& rhs) = default;
682 BoxArray (BoxArray&& rhs) noexcept = default;
683 BoxArray& operator= (BoxArray const& rhs) = default;
684 BoxArray& operator= (BoxArray&& rhs) noexcept = default;
685 ~BoxArray() noexcept = default;
686
692 explicit BoxArray (const Box& bx);
693
697 explicit BoxArray (size_t n);
698
705 BoxArray (const Box* bxvec,
706 int nbox);
707
709 explicit BoxArray (const BoxList& bl);
710 explicit BoxArray (BoxList&& bl) noexcept;
711
718 BoxArray (const BoxArray& rhs, const BATransformer& trans);
719
726 BoxArray (BoxList&& bl, IntVect const& max_grid_size);
727
735 void define (const Box& bx);
743 void define (const BoxList& bl);
744 void define (BoxList&& bl) noexcept;
745
747 void clear ();
748
750 void resize (Long len);
751
753 [[nodiscard]] Long size () const noexcept { return m_ref->m_abox.size(); }
754
756 [[nodiscard]] Long capacity () const noexcept { return static_cast<Long>(m_ref->m_abox.capacity()); }
757
759 [[nodiscard]] bool empty () const noexcept { return m_ref->m_abox.empty(); }
760
762 [[nodiscard]] Long numPts() const noexcept;
763
765 [[nodiscard]] double d_numPts () const noexcept;
776 int readFrom (std::istream& is);
777
779 std::ostream& writeOn (std::ostream&) const;
780
782 [[nodiscard]] bool operator== (const BoxArray& rhs) const noexcept;
783
785 [[nodiscard]] bool operator!= (const BoxArray& rhs) const noexcept;
786
787 [[nodiscard]] bool operator== (const Vector<Box>& bv) const noexcept;
788 [[nodiscard]] bool operator!= (const Vector<Box>& bv) const noexcept;
789
791 [[nodiscard]] bool CellEqual (const BoxArray& rhs) const noexcept;
792
801 BoxArray& maxSize (int block_size);
802
812 BoxArray& maxSize (const IntVect& block_size);
813
824 BoxArray& minmaxSize (const IntVect& min_size, const IntVect& max_size);
825
834 BoxArray& refine (int refinement_ratio);
835
844 BoxArray& refine (const IntVect& iv);
845
854 BoxArray& coarsen (int refinement_ratio);
855
868 [[nodiscard]] bool coarsenable (int refinement_ratio, int min_width=1) const;
881 [[nodiscard]] bool coarsenable (const IntVect& refinement_ratio, int min_width=1) const;
894 [[nodiscard]] bool coarsenable (const IntVect& refinement_ratio, const IntVect& min_width) const;
895
904 BoxArray& coarsen (const IntVect& iv);
905
916 BoxArray& growcoarsen (int n, const IntVect& iv);
927 BoxArray& growcoarsen (IntVect const& ngrow, const IntVect& iv);
928
937 BoxArray& grow (int n);
938
947 BoxArray& grow (const IntVect& iv);
957 BoxArray& grow (int idir, int n_cell);
967 BoxArray& growLo (int idir, int n_cell);
977 BoxArray& growHi (int idir, int n_cell);
993 BoxArray& surroundingNodes (int dir);
994
1001
1009 BoxArray& enclosedCells (int dir);
1010
1018 BoxArray& convert (IndexType typ);
1019
1027 BoxArray& convert (const IntVect& iv);
1028
1037 BoxArray& convert (Box (*fp)(const Box&));
1038
1048 BoxArray& shift (int dir, int nzones);
1049
1058 BoxArray& shift (const IntVect &iv);
1059
1069 void set (int i, const Box& ibox);
1070
1072 [[nodiscard]] Box operator[] (int index) const noexcept {
1073 return m_bat(m_ref->m_abox[index]);
1074 }
1075
1077 [[nodiscard]] Box operator[] (const MFIter& mfi) const noexcept;
1078
1080 [[nodiscard]] Box get (int index) const noexcept { return operator[](index); }
1081
1083 [[nodiscard]] Box getCellCenteredBox (int index) const noexcept {
1084 return m_bat.coarsen(m_ref->m_abox[index]);
1085 }
1086
1093 [[nodiscard]] bool ok () const;
1094
1096 [[nodiscard]] bool isDisjoint () const;
1097
1099 [[nodiscard]] BoxList boxList () const;
1100
1105 [[nodiscard]] bool contains (const IntVect& v) const;
1106
1116 [[nodiscard]]
1117 bool contains (const Box& b, bool assume_disjoint_ba = false,
1118 const IntVect& ng = IntVect(0)) const;
1119
1127 [[nodiscard]]
1128 bool contains (const BoxArray& ba, bool assume_disjoint_ba = false,
1129 const IntVect& ng = IntVect(0)) const;
1130
1140 [[nodiscard]]
1141 bool contains (const BoxArray& ba, Periodicity const& period) const;
1142
1144 [[nodiscard]] Box minimalBox () const;
1149 [[nodiscard]] Box minimalBox (Long& npts_avg_box) const;
1150
1156 [[nodiscard]]
1157 bool intersects (const Box& b, int ng = 0) const;
1158
1164 [[nodiscard]]
1165 bool intersects (const Box& b, const IntVect& ng) const;
1166
1173 [[nodiscard]]
1174 std::vector< std::pair<int,Box> > intersections (const Box& bx) const;
1175
1182 [[nodiscard]]
1183 std::vector< std::pair<int,Box> > intersections (const Box& bx, bool first_only, int ng) const;
1184
1191 [[nodiscard]]
1192 std::vector< std::pair<int,Box> > intersections (const Box& bx, bool first_only, const IntVect& ng) const;
1193
1199 void intersections (const Box& bx, std::vector< std::pair<int,Box> >& isects) const;
1200
1208 void intersections (const Box& bx, std::vector< std::pair<int,Box> >& isects,
1209 bool first_only, int ng) const;
1210
1218 void intersections (const Box& bx, std::vector< std::pair<int,Box> >& isects,
1219 bool first_only, const IntVect& ng) const;
1220
1222 [[nodiscard]] BoxList complementIn (const Box& b) const;
1223 void complementIn (BoxList& bl, const Box& b) const;
1224
1226 [[nodiscard]] BoxList complementIn (const Box& b, const Periodicity& period) const;
1227
1229 void clear_hash_bin () const;
1230
1232 void removeOverlap (bool simplify=true);
1233
1235 [[nodiscard]] static bool SameRefs (const BoxArray& lhs, const BoxArray& rhs) { return lhs.m_ref == rhs.m_ref; }
1236
1237 struct RefID {
1238 RefID () noexcept = default;
1239 explicit RefID (BARef* data_) noexcept : data(data_) {}
1240 bool operator< (const RefID& rhs) const noexcept { return std::less<>()(data,rhs.data); }
1241 bool operator== (const RefID& rhs) const noexcept { return data == rhs.data; }
1242 bool operator!= (const RefID& rhs) const noexcept { return data != rhs.data; }
1243 friend std::ostream& operator<< (std::ostream& os, const RefID& id);
1244 private:
1245 BARef* data{nullptr};
1246 };
1247
1249 [[nodiscard]] RefID getRefID () const noexcept { return RefID { m_ref.get() }; }
1250
1252 [[nodiscard]] IndexType ixType () const noexcept { return m_bat.index_type(); }
1253
1255 [[nodiscard]] IntVect crseRatio () const noexcept { return m_bat.coarsen_ratio(); }
1256
1257 static void Initialize ();
1258 static void Finalize ();
1259 static bool initialized;
1260
1262 void uniqify ();
1263
1265 [[nodiscard]] BoxList const& simplified_list () const;
1267 [[nodiscard]] BoxArray simplified () const;
1268
1270 [[nodiscard]] BATransformer const& transformer () const;
1271
1273 [[nodiscard]] std::weak_ptr<BARef> getWeakRef () const;
1275 [[nodiscard]] std::shared_ptr<BARef> const& getSharedRef () const;
1277 std::shared_ptr<BARef>& getSharedRef ();
1278
1279 friend class AmrMesh;
1280 friend class FabArrayBase;
1281
1282private:
1284 void type_update ();
1285
1286 [[nodiscard]] BARef::HashType& getHashMap () const;
1287
1288 [[nodiscard]] IntVect getDoiLo () const noexcept;
1289 [[nodiscard]] IntVect getDoiHi () const noexcept;
1290
1291 BATransformer m_bat;
1293 std::shared_ptr<BARef> m_ref;
1294 mutable std::shared_ptr<BoxList> m_simplified_list;
1295};
1296
1298std::ostream& operator<< (std::ostream& os, const BoxArray& ba);
1299
1300std::ostream& operator<< (std::ostream& os, const BoxArray::RefID& id);
1301
1302}
1303
1304#endif /*BL_BOXARRAY_H*/
Fixed-size array types for use on GPU and CPU.
A list of Boxes sharing a common IndexType.
int idir
Definition AMReX_HypreMLABecLap.cpp:1133
#define AMREX_D_DECL(a, b, c)
Definition AMReX_SPACE.H:171
Definition AMReX_AmrMesh.H:69
Reference-counted collection of Boxes.
Definition AMReX_BoxArray.H:676
IndexType ixType() const noexcept
Return index type of this BoxArray.
Definition AMReX_BoxArray.H:1252
Box operator[](int index) const noexcept
Return element index of this BoxArray.
Definition AMReX_BoxArray.H:1072
Long capacity() const noexcept
Return the number of boxes that can be held in the current allocated storage.
Definition AMReX_BoxArray.H:756
RefID getRefID() const noexcept
Return a unique ID of the reference.
Definition AMReX_BoxArray.H:1249
bool intersects(const Box &b, int ng=0) const
True if b intersects this BoxArray(+ng).
Definition AMReX_BoxArray.cpp:1169
BoxArray & grow(int n)
Grow each Box by n cells in every direction.
Definition AMReX_BoxArray.cpp:706
static void Finalize()
Definition AMReX_BoxArray.cpp:282
BoxArray() noexcept
Construct an empty BoxArray.
Definition AMReX_BoxArray.cpp:287
std::vector< std::pair< int, Box > > intersections(const Box &bx) const
Return all intersections of bx with this BoxArray.
Definition AMReX_BoxArray.cpp:1186
std::ostream & writeOn(std::ostream &) const
Output this BoxArray to a checkpoint file.
Definition AMReX_BoxArray.cpp:482
bool contains(const IntVect &v) const
Return true if the cell v is within any box in the array.
Definition AMReX_BoxArray.cpp:977
static void Initialize()
Definition AMReX_BoxArray.cpp:271
BoxList const & simplified_list() const
Return (and cache) a simplified BoxList representation; regular AMR grids only.
Definition AMReX_BoxArray.cpp:1636
BoxList boxList() const
Create a BoxList from this BoxArray.
Definition AMReX_BoxArray.cpp:949
BoxArray simplified() const
Return a simplified BoxArray; regular AMR grids only.
Definition AMReX_BoxArray.cpp:1647
IntVect crseRatio() const noexcept
Return crse ratio of this BoxArray.
Definition AMReX_BoxArray.H:1255
int readFrom(std::istream &is)
Initialize the BoxArray from the supplied istream.
Definition AMReX_BoxArray.cpp:468
void define(const Box &bx)
Initialize the BoxArray from a single box.
Definition AMReX_BoxArray.cpp:352
Long numPts() const noexcept
Returns the total number of cells contained in all boxes in the BoxArray.
Definition AMReX_BoxArray.cpp:394
void clear_hash_bin() const
Clear out the internal hash table used by intersections.
Definition AMReX_BoxArray.cpp:1440
Box minimalBox() const
Return smallest Box that contains all Boxes in this BoxArray.
Definition AMReX_BoxArray.cpp:1067
BoxArray & minmaxSize(const IntVect &min_size, const IntVect &max_size)
Constrain Box sizes between min_size and max_size.
Definition AMReX_BoxArray.cpp:577
BoxArray & refine(int refinement_ratio)
Refine each Box in the BoxArray by refinement_ratio.
Definition AMReX_BoxArray.cpp:593
std::weak_ptr< BARef > getWeakRef() const
Return a weak pointer to the shared Box storage.
Definition AMReX_BoxArray.cpp:1659
bool coarsenable(int refinement_ratio, int min_width=1) const
Check whether every Box is coarsenable by refinement_ratio.
Definition AMReX_BoxArray.cpp:615
BoxArray & growLo(int idir, int n_cell)
Grow each Box on the lower side of dimension idir.
Definition AMReX_BoxArray.cpp:752
void resize(Long len)
Resize the BoxArray. See Vector<T>::resize() for the gory details.
Definition AMReX_BoxArray.cpp:387
BoxArray & enclosedCells()
Convert the BoxArray to cell-centered type in all directions.
Definition AMReX_BoxArray.cpp:799
std::shared_ptr< BARef > const & getSharedRef() const
Return the shared pointer to the Box storage.
Definition AMReX_BoxArray.cpp:1665
Box getCellCenteredBox(int index) const noexcept
Return cell-centered box at element index of this BoxArray.
Definition AMReX_BoxArray.H:1083
double d_numPts() const noexcept
Returns the total number of cells (in double type) contained in all boxes in the BoxArray.
Definition AMReX_BoxArray.cpp:431
BATransformer const & transformer() const
Return the transformer describing index-type/coarsening metadata.
Definition AMReX_BoxArray.cpp:1653
static bool initialized
Definition AMReX_BoxArray.H:1259
static bool SameRefs(const BoxArray &lhs, const BoxArray &rhs)
whether two BoxArrays share the same data
Definition AMReX_BoxArray.H:1235
BoxArray & growHi(int idir, int n_cell)
Grow each Box on the upper side of dimension idir.
Definition AMReX_BoxArray.cpp:768
BoxList complementIn(const Box &b) const
Return box - boxarray.
Definition AMReX_BoxArray.cpp:1314
void removeOverlap(bool simplify=true)
Change the BoxArray to one with no overlap and then simplify it (see the simplify function in BoxList...
Definition AMReX_BoxArray.cpp:1456
BoxArray & coarsen(int refinement_ratio)
Coarsen each Box in the BoxArray by refinement_ratio.
Definition AMReX_BoxArray.cpp:672
BoxArray & shift(int dir, int nzones)
Shift each Box in the BoxArray by nzones cells in direction dir.
Definition AMReX_BoxArray.cpp:847
BoxArray & surroundingNodes()
Convert the BoxArray to nodal type in all directions.
Definition AMReX_BoxArray.cpp:784
BoxArray(BoxArray &&rhs) noexcept=default
Long size() const noexcept
Return the number of boxes in the BoxArray.
Definition AMReX_BoxArray.H:753
Box get(int index) const noexcept
Return element index of this BoxArray.
Definition AMReX_BoxArray.H:1080
~BoxArray() noexcept=default
bool CellEqual(const BoxArray &rhs) const noexcept
Are the BoxArrays equal after conversion to cell-centered.
Definition AMReX_BoxArray.cpp:546
void set(int i, const Box &ibox)
Set element i to ibox if no coarsening transformer is active.
Definition AMReX_BoxArray.cpp:878
BoxArray & growcoarsen(int n, const IntVect &iv)
Expand boxes by n cells in every direction, then coarsen by iv.
Definition AMReX_BoxArray.cpp:685
BoxArray & convert(IndexType typ)
Set the IndexType of the BoxArray.
Definition AMReX_BoxArray.cpp:813
bool ok() const
Return true if the BoxArray is empty or every Box reports ok().
Definition AMReX_BoxArray.cpp:894
BoxArray & operator=(BoxArray const &rhs)=default
void uniqify()
Make ourselves unique.
Definition AMReX_BoxArray.cpp:1613
BoxArray(const BoxArray &rhs)=default
bool empty() const noexcept
Return whether the BoxArray is empty.
Definition AMReX_BoxArray.H:759
bool isDisjoint() const
Return true if set of intersecting Boxes in BoxArray is null.
Definition AMReX_BoxArray.cpp:920
BoxArray & maxSize(int block_size)
Split any Box whose side length exceeds block_size.
Definition AMReX_BoxArray.cpp:553
void clear()
Remove all Boxes from the BoxArray.
Definition AMReX_BoxArray.cpp:379
A list of Boxes sharing a common IndexType.
Definition AMReX_BoxList.H:109
Base class for FabArray.
Definition AMReX_FabArrayBase.H:42
Iterator for looping ever tiles and boxes of amrex::FabArray based containers.
Definition AMReX_MFIter.H:88
This provides length of period for periodic domains. 0 means it is not periodic in that direction....
Definition AMReX_Periodicity.H:17
This class is a thin wrapper around std::vector. Unlike vector, Vector::operator[] provides bound che...
Definition AMReX_Vector.H:29
amrex_long Long
Definition AMReX_INT.H:30
__host__ __device__ BoxND< dim > convert(const BoxND< dim > &b, const IntVectND< dim > &typ) noexcept
Return a copy of b converted to the nodal flags typ.
Definition AMReX_Box.H:1630
__host__ __device__ BoxND< dim > coarsen(const BoxND< dim > &b, int ref_ratio) noexcept
Return a copy of b coarsened by the isotropic ratio ref_ratio.
Definition AMReX_Box.H:1469
__host__ __device__ BoxND< dim > refine(const BoxND< dim > &b, int ref_ratio) noexcept
Return a copy of b refined by the isotropic ratio ref_ratio.
Definition AMReX_Box.H:1510
Definition AMReX_Amr.cpp:50
BoxArray intersect(const BoxArray &ba, const Box &b, int ng)
Make a BoxArray from the intersection of Box b and BoxArray(+ghostcells).
Definition AMReX_BoxArray.cpp:1717
AMReX * Initialize(MPI_Comm mpi_comm, std::ostream &a_osout=std::cout, std::ostream &a_oserr=std::cerr, ErrorHandler a_errhandler=nullptr, int a_device_id=-1)
Initialize AMReX using an existing MPI communicator.
Definition AMReX.cpp:343
BoxND< 3 > Box
Box is an alias for amrex::BoxND instantiated with AMREX_SPACEDIM.
Definition AMReX_BaseFwd.H:35
BoxList GetBndryCells(const BoxArray &ba, int ngrow)
Find the boundary/ghost region around a BoxArray.
Definition AMReX_BoxArray.cpp:1842
IndexTypeND< 3 > IndexType
IndexType is an alias for amrex::IndexTypeND instantiated with AMREX_SPACEDIM.
Definition AMReX_BaseFwd.H:41
void Finalize(AMReX *pamrex)
Finalize AMReX state associated with the supplied instance pointer.
Definition AMReX.cpp:824
BoxArray decompose(Box const &domain, int nboxes, Array< bool, 3 > const &decomp, bool no_overlap)
Decompose domain box into BoxArray.
Definition AMReX_BoxArray.cpp:1943
bool match(const BoxArray &x, const BoxArray &y)
Return true if two BoxArrays cover the same index space.
Definition AMReX_BoxArray.cpp:1930
BoxArray complementIn(const Box &b, const BoxArray &ba)
Make a BoxArray from the complement of ba within the Box b.
Definition AMReX_BoxArray.cpp:1710
IntVectND< 3 > IntVect
IntVect is an alias for amrex::IntVectND instantiated with AMREX_SPACEDIM.
Definition AMReX_BaseFwd.H:38
void readBoxArray(BoxArray &ba, std::istream &is, bool bReadSpecial)
Read a BoxArray from a stream.
Definition AMReX_BoxArray.cpp:1900
BATransformer BndryBATransformer
Definition AMReX_BoxArray.H:659
BoxArray boxComplement(const Box &b1in, const Box &b2)
Make a BoxArray from the complement of b2 in b1in.
Definition AMReX_BoxArray.cpp:1703
Definition AMReX_BoxArray.H:1237
friend std::ostream & operator<<(std::ostream &os, const RefID &id)
Definition AMReX_BoxArray.cpp:2126
RefID() noexcept=default
bool operator<(const RefID &rhs) const noexcept
Definition AMReX_BoxArray.H:1240
bool operator!=(const RefID &rhs) const noexcept
Definition AMReX_BoxArray.H:1242
bool operator==(const RefID &rhs) const noexcept
Definition AMReX_BoxArray.H:1241